# tests.syncdaemon.test_status_listener
#
# Author: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the syncdaemon status listener."""

import os

from twisted.internet import defer
from twisted.trial.unittest import TestCase

from contrib.testing.testcase import FakeMainTestCase
from ubuntuone.syncdaemon import status_listener
from ubuntuone.syncdaemon.volume_manager import Share


class GetListenerTestCase(TestCase):
    """The status listener is created."""

    def test_returns_listener(self):
        """get_listener returns a listener if status reporting is enabled."""
        self.patch(status_listener, "should_start_listener", lambda: True)
        fsm = object()
        vm = object()
        listener = status_listener.get_listener(fsm, vm)
        self.assertIsInstance(listener, status_listener.StatusListener)
        self.assertEqual(listener.fsm, fsm)
        self.assertEqual(listener.vm, vm)
        self.assertNotEqual(listener.aggregator, None)

    def test_zeitgeist_not_installed_returns_none(self):
        """get_listener returns None if status reporting is disabled."""
        self.patch(status_listener, "should_start_listener", lambda: False)
        listener = status_listener.get_listener(None, None)
        self.assertEqual(listener, None)


def listen_for(event_q, event, callback, count=1, collect=False):
    """Setup a EQ listener for the specified event."""
    class Listener(object):
        """A basic listener to handle the pushed event."""

        def __init__(self):
            self.hits = 0
            self.events = []

        def _handle_event(self, *args, **kwargs):
            self.hits += 1
            if collect:
                self.events.append((args, kwargs))
            if self.hits == count:
                event_q.unsubscribe(self)
                if collect:
                    callback(self.events)
                elif kwargs:
                    callback((args, kwargs))
                else:
                    callback(args)

    listener = Listener()
    setattr(listener, 'handle_'+event, listener._handle_event)
    event_q.subscribe(listener)
    return listener


class FakeStatusAggregator(object):
    """A fake status aggregator."""

    def __init__(self):
        """Initialize this instance."""
        self.call_log = []

    def __getattr__(self, f_name):
        """Return a method that will log it's args when called."""

        def f(*args, **kwargs):
            """Log the args I'm passed."""
            self.call_log.append((f_name, args, kwargs))

        return f


class StatusListenerTestCase(FakeMainTestCase):
    """Tests for StatusListener."""

    def setUp(self):
        """Initialize this instance."""
        super(StatusListenerTestCase, self).setUp()
        self.aggregator = FakeStatusAggregator()
        self.listener = status_listener.StatusListener(self.fs, self.vm,
                                                       self.aggregator)
        self.event_q.subscribe(self.listener)

    def _listen_for(self, *args, **kwargs):
        return listen_for(self.main.event_q, *args, **kwargs)


class PublicFilesStatusTestCase(StatusListenerTestCase):
    """Public files events are passed to the status object."""

    @defer.inlineCallbacks
    def test_publish_url_is_forwarded(self):
        """Publishing a file with a url is forwarded."""
        share_id = "share"
        node_id = "node_id"
        is_public = True
        public_url = 'http://example.com/foo.mp3'

        share_path = os.path.join(self.shares_dir, 'share')
        self.main.vm.add_share(Share(path=share_path, volume_id='share',
                               other_username='other username'))
        path = os.path.join(share_path, "foo.mp3")
        self.main.fs.create(path, str(share_id))
        self.main.fs.set_node_id(path, str(node_id))

        d = defer.Deferred()
        self._listen_for('AQ_CHANGE_PUBLIC_ACCESS_OK', d.callback)
        self.main.event_q.push('AQ_CHANGE_PUBLIC_ACCESS_OK',
                               share_id=share_id, node_id=node_id,
                               is_public=is_public, public_url=public_url)
        yield d
        call = ("file_published", (public_url,), {})
        self.assertIn(call, self.aggregator.call_log)

    @defer.inlineCallbacks
    def test_unpublish_url_is_forwarded(self):
        """Unpublishing a file with a url is forwarded."""
        share_id = "share"
        node_id = "node_id"
        is_public = False
        public_url = 'http://example.com/foo.mp3'

        share_path = os.path.join(self.shares_dir, 'share')
        self.main.vm.add_share(Share(path=share_path, volume_id='share',
                               other_username='other username'))
        path = os.path.join(share_path, "foo.mp3")
        self.main.fs.create(path, str(share_id))
        self.main.fs.set_node_id(path, str(node_id))

        d = defer.Deferred()
        self._listen_for('AQ_CHANGE_PUBLIC_ACCESS_OK', d.callback)
        self.main.event_q.push('AQ_CHANGE_PUBLIC_ACCESS_OK',
                               share_id=share_id, node_id=node_id,
                               is_public=is_public, public_url=public_url)
        yield d
        call = ("file_unpublished", (public_url,), {})
        self.assertIn(call, self.aggregator.call_log)
